/*
 * SingletonWithAspectJ - Singletons with AspectJ
 * Copyright (C) 2007 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.aspectj.singleton;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;

@Aspect
public class SingletonAspect {

	private static final Logger log = Logger.getLogger("singleton");
	private final Map<Class<?>, Object> singletons;

	public SingletonAspect() {
		this.singletons = new HashMap<Class<?>, Object>();
	}

	/**
	 * All singletons are defined in this pointcut.
	 */
	@Pointcut("call(*.SingletonAop.new(..))")
	protected void singletons() {
	}

	/**
	 * Circumvents the call to "new" if an instance of that class already exists.
	 */
	@Around("singletons()")
	public Object aroundSingletons(final ProceedingJoinPoint thisJoinPoint) throws Throwable {
		final Class<?> singletonClass = thisJoinPoint.getSignature().getDeclaringType();
		log.debug("Class: " + singletonClass);
		Object singletonObject = this.singletons.get(singletonClass);
		if (singletonObject == null) {
			singletonObject = thisJoinPoint.proceed();
			this.singletons.put(singletonClass, singletonObject);
		}
		log.debug("Object: " + singletonObject);
		return singletonObject;
	}
}